//
//  CommentListCell.swift
//  BJVideoPlayerUI
//
//  Created by ney on 2023/6/5.
//  Copyright © 2023 BaijiaYun. All rights reserved.
//

import UIKit
import BJVideoPlayerCore

public class CommentListCell: UITableViewCell {
    // MARK: - api
    var deleteActionHandler: ((CommentListCell, BJVCommentItem?) -> Void)?
    var likeActionHandler: ((CommentListCell, BJVCommentItem?, Bool) -> Void)?
    
    override public init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
        super.init(style: style, reuseIdentifier: reuseIdentifier)
        self.selectionStyle = .none
        makeSubViews()
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    @objc public func update(model: BJVCommentItem) {
        let avatar = model.userAvatar ?? ""
        if avatar.isEmpty {
            avatarImageView.image = UIImage.bjpu_imageNamed("comment_cell_empty_avatar")
        }
        else {
            avatarImageView.bjl_setImage(with: URL(string: avatar))
        }
        topCommentTagImageView.isHidden = !model.top
        commentTextLabel.attributedText = buildCommentAttributedTextWithName(model.userName ?? "", comment: model.comment ?? "", top: model.top)
        dateLabel.text = model.updateTime
        deleteButton.isHidden = !model.createdBySelf
        likeButton.isSelected = model.like
        data = model
    }
    
    // MARK: - private var
    private let avatarImageView = UIImageView(),
                commentTextLabel = UILabel(),
                topCommentTagImageView = UIImageView(),
                dateLabel = UILabel()
    private weak var data: BJVCommentItem?
    
    private lazy var deleteButton: UIButton = {
        let button = UIButton()
        button.bjl_setImage(UIImage.bjpu_imageNamed("comment_delete_normal"), for: .normal)
        button.bjl_addHandler { [unowned self] button in
            deleteActionHandler?(self, data)
        }
        return button
    }()
    private lazy var likeButton: UIButton = {
        let button = UIButton()
        button.bjl_setImage(UIImage.bjpu_imageNamed("comment_like_selected"), for: .selected)
        button.bjl_setImage(UIImage.bjpu_imageNamed("comment_like_normal"), for: .normal)
        button.bjl_addHandler { [unowned self] button in
            likeActionHandler?(self, data, !button.isSelected)
        }
        return button
    }()
    let bottomActionStackView: UIStackView = {
        let stack = UIStackView()
        stack.axis = .horizontal
        stack.alignment = .center
        stack.spacing = 20
        return stack
    }()
    let separateLineView: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor.bjl_color(withHexString: "#EEEEEE")
        return view
    }()
}

fileprivate extension CommentListCell {
    func makeSubViews() {
        avatarImageView.layer.cornerRadius = 18
        avatarImageView.clipsToBounds = true

        commentTextLabel.textColor = .bjl_color(hex: 0x333333)
        commentTextLabel.font = .systemFont(ofSize: 14)
        commentTextLabel.textAlignment = .left
        commentTextLabel.numberOfLines = 0
        
        topCommentTagImageView.image = UIImage.bjpu_imageNamed("comment_cell_top")
        
        dateLabel.textColor = .bjl_color(hex: 0x666666)
        dateLabel.font = .systemFont(ofSize: 12)
        
        bottomActionStackView.addArrangedSubview(deleteButton)
        bottomActionStackView.addArrangedSubview(likeButton)
        
        avatarImageView.layout(in: contentView) { make in
            make.left.equalToSuperview().offset(12)
            make.top.equalToSuperview().offset(16)
            make.bottom.lessThanOrEqualToSuperview().offset(-16)
            make.width.height.equalTo(36)
        }
        commentTextLabel.layout(in: contentView) { make in
            make.top.equalTo(avatarImageView).offset(2)
            make.left.equalTo(avatarImageView.snp.right).offset(14)
            make.right.equalToSuperview().offset(-12)
        }
        topCommentTagImageView.layout(in: contentView) { make in
            make.left.equalTo(commentTextLabel)
            make.top.equalTo(avatarImageView).offset(1)
        }
        dateLabel.layout(in: contentView) { make in
            make.left.equalTo(commentTextLabel)
            make.top.equalTo(commentTextLabel.snp.bottom).offset(12)
            make.right.lessThanOrEqualTo(commentTextLabel)
            make.bottom.equalToSuperview().offset(-16)
        }
        bottomActionStackView.layout(in: contentView) { make in
            make.right.equalToSuperview().offset(-12)
            make.centerY.equalTo(dateLabel)
        }
        separateLineView.layout(in: contentView) { make in
            make.left.equalTo(commentTextLabel)
            make.right.equalTo(bottomActionStackView)
            make.bottom.equalToSuperview()
            make.height.equalTo(1)
        }
    }
    
    func buildCommentAttributedTextWithName(_ name: String, comment: String, top: Bool) -> NSAttributedString {
        var name = name
        if top { name = "            " + name }
        let p1 = (name + ": ", UIFont.systemFont(ofSize: 14, weight: .regular), UIColor.bjl_color(withHexString: "#1E5EFF"))
        let p2 = (comment, UIFont.systemFont(ofSize: 14, weight: .regular), UIColor.bjl_color(withHexString: "#333333"))
        
        let result = [p1, p2].reduce(NSMutableAttributedString(string: "")) { partialResult, i in
            let str = NSMutableAttributedString(string: i.0)
            
            str.addAttribute(.font, value: i.1, range: NSRange(location: 0, length: str.length))
            str.addAttribute(.foregroundColor, value: i.2 ?? UIColor.black, range: NSRange(location: 0, length: str.length))
            partialResult.append(str)
            return partialResult
        }
        
        let paragraphStyle = NSMutableParagraphStyle()
        paragraphStyle.lineSpacing = 6
        result.addAttribute(.paragraphStyle, value: paragraphStyle, range: NSRange(location: 0, length: result.length))
        
        return result
    }
}
