//
//  BJPUMediaControlView.m
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/3/8.
//

#if __has_feature(modules) && BJL_USE_SEMANTIC_IMPORT
@import BJLiveBase;
#else
#import <BJLiveBase/BJLiveBase.h>
#endif

#import "BJPUMediaControlView.h"
#import "BJPUProgressView.h"
#import "BJPUTheme.h"
#import "BJPUAppearance.h"

static const CGFloat controlButtonH = 30.0;
static const CGFloat timeFontSize = 14.0;

@interface BJPUMediaControlView ()

@property (nonatomic, weak) BJVPlayerManager *playerManager;

@property (nonatomic) UIStackView *buttonStack;
@property (nonatomic) UIButton *playButton;
@property (nonatomic) UIButton *pauseButton;
@property (nonatomic) UIButton *scaleButton;
@property (nonatomic) UIButton *subtitleButton;
@property (nonatomic) UIButton *definitionButton;
@property (nonatomic) UIButton *albumButton;
@property (nonatomic) UIButton *rateButton;
@property (nonatomic) UIButton *preButton, *nextButton;

@property (nonatomic) BOOL scaleButtonEnable;
@property (nonatomic) BOOL rateButtonEnable;
@property (nonatomic) BOOL externalAlnumShow;

@property (nonatomic) NSTimer *timer;
@property (nonatomic, readwrite) BOOL controlsHidden;

@property (nonatomic) UILabel *currentTimeLabel;
@property (nonatomic) BJPUProgressView *progressView;
@property (nonatomic) BOOL stopUpdateProgress;
@property (nonatomic, readwrite) BOOL existSubtitle;
@property (nonatomic) BOOL horizen;

@end

@implementation BJPUMediaControlView

- (instancetype)initWithPlayerManager:(BJVPlayerManager *)playerManager {
    self = [super init];
    if (self) {
        self.scaleButtonEnable = YES;
        self.rateButtonEnable = YES;
        self.playerManager = playerManager;
        self.backgroundColor = [UIColor clearColor];
        [self setupSubviews];
        [self makeObserving];
        [self autoHideControls];
    }
    return self;
}

#pragma mark - subViews

- (void)setupSubviews {
    CGFloat margin = 10.0;
    CGFloat progressViewHeight = 36.0;

    UIView *bgview = [[UIView alloc] init];
    bgview.backgroundColor = [[BJPUTheme brandColor] colorWithAlphaComponent:0.4];
    [self addSubview:bgview];
    [bgview bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.left.bottom.right.equalTo(self);
        make.top.equalTo(self).offset(progressViewHeight / 2);
    }];

    // 播放按钮
    [self addSubview:self.preButton];
    [self addSubview:self.nextButton];
    self.preButton.hidden = YES;
    self.nextButton.hidden = YES;
    
    [self addSubview:self.playButton];
    [self.playButton bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.equalTo(self).offset(margin);
        make.bottom.equalTo(self).offset(-8);
        make.size.equal.sizeOffset(CGSizeMake(controlButtonH, controlButtonH));
    }];

    // 暂停按钮
    [self addSubview:self.pauseButton];
    [self.pauseButton bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.edges.equalTo(self.playButton);
    }];
    
    [self addSubview:self.buttonStack];

    // 进度条: 约束待定
    [self addSubview:self.progressView];

    // 当前时间: 约束待定
    [self addSubview:self.currentTimeLabel];
    
    [self.currentTimeLabel bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.equalTo(self.playButton.bjl_right).offset(5.0);
        make.centerY.equalTo(self.playButton);
        make.horizontal.compressionResistance.hugging.required();
    }];
    [self.progressView bjl_makeConstraints:^(BJLConstraintMaker *make) {
        make.left.right.top.equalTo(self);
        make.height.equalTo(@(progressViewHeight));
    }];

    [self.buttonStack bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.centerY.equalTo(self.pauseButton);
        make.right.equalTo(self).offset(-margin);
        make.height.equalTo(@(controlButtonH));
    }];

    NSArray<UIButton *> *buttonArray = @[self.scaleButton, self.rateButton, self.albumButton, self.definitionButton, self.subtitleButton];
    for (UIButton *b in buttonArray) {
        [self.buttonStack insertArrangedSubview:b atIndex:0];
    }
}

- (void)makeObserving {
    bjl_weakify(self);
    [self bjl_kvo:BJLMakeProperty(self, controlsHidden)
           filter:^BOOL(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        return [value integerValue] != [oldValue integerValue];
    }
         observer:^BJLControlObserving(id _Nullable value, id _Nullable oldValue, BJLPropertyChange *_Nullable change) {
        bjl_strongify(self);
        if (![value boolValue]) {
            [self autoHideControls];
        }
        return YES;
    }];
}

- (void)autoHideControls {
    if (!self.horizen) {
        return;
    }
    
    if (self.timer.isValid) {
        [self.timer invalidate];
    }

    bjl_weakify(self);
    __block int time = 5;
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1 repeats:YES block:^(NSTimer *_Nonnull timer) {
        bjl_strongify(self);
        if (time == 0) {
            [self setControlsHidden:YES];
            [timer invalidate];
        }
        time--;
    }];
    [self.timer fire];
}

- (void)setControlsHidden:(BOOL)hidden {
    _controlsHidden = hidden;
    self.hidden = hidden;
}

- (void)showOrHideControls {
    if (self.hidden) {
        self.controlsHidden = NO;
    }
    else {
        _controlsHidden = YES;
        self.hidden = YES;
    }
}

#pragma mark - public

- (void)updateConstraintsWithLayoutType:(BOOL)horizon {
    self.horizen = horizon;
    
    if (self.horizen) {
        [self.scaleButton setImage:[UIImage bjpu_imageNamed:@"ic_scale_horizen"] forState:UIControlStateNormal];
    }
    else {
        [self.scaleButton setImage:[UIImage bjpu_imageNamed:@"ic_scale"] forState:UIControlStateNormal];
    }

    self.rateButton.hidden = !self.rateButtonEnable;
    self.subtitleButton.hidden = !self.existSubtitle || !self.horizen;
    [self updateAlbumButtonHidden:(self.preButton.hidden || !self.horizen) && !self.externalAlnumShow];
    [self autoHideControls];
}

- (void)updateProgressWithCurrentTime:(NSTimeInterval)currentTime
                        cacheDuration:(NSTimeInterval)cacheDuration
                        totalDuration:(NSTimeInterval)totalDuration {
    if (self.stopUpdateProgress) {
        return;
    }
    BOOL durationInvalid = (ceil(totalDuration) <= 0);
    NSString *duration = durationInvalid ? @"-" : [NSString stringWithFormat:@"%@", [NSString stringFromTimeInterval:totalDuration]];
    NSString *currentTimeString = durationInvalid ? @"-" : [NSString stringFromTimeInterval:currentTime totalTimeInterval:totalDuration];
    self.currentTimeLabel.text = [NSString stringWithFormat:@"%@/%@", currentTimeString, duration];

    [self.progressView setValue:currentTime
                          cache:cacheDuration
                       duration:totalDuration];
}

- (void)updateWithPlayState:(BOOL)playing {
    self.playButton.hidden = playing;
    self.pauseButton.hidden = !playing;
}

- (void)setSlideEnable:(BOOL)enable {
    self.progressView.userInteractionEnabled = enable;
}

- (void)setSwitchVideoEnable:(BOOL)enable {
    self.preButton.hidden = !enable;
    self.nextButton.hidden = !enable;
    CGFloat margin = 10.0;
    
    UIButton *currentLeftButton = self.playButton;
    if (enable) {
        [self.preButton bjl_remakeConstraints:^(BJLConstraintMaker *make) {
            make.left.equalTo(self).offset(margin);
            make.bottom.equalTo(self).offset(-8);
            make.size.equal.sizeOffset(CGSizeMake(controlButtonH, controlButtonH));
        }];
        [self.playButton bjl_remakeConstraints:^(BJLConstraintMaker *make) {
            make.left.equalTo(self.preButton.bjl_right).offset(margin);
            make.bottom.equalTo(self).offset(-8);
            make.size.equal.sizeOffset(CGSizeMake(controlButtonH, controlButtonH));
        }];
        [self.nextButton bjl_remakeConstraints:^(BJLConstraintMaker *make) {
            make.left.equalTo(self.playButton.bjl_right).offset(margin);
            make.bottom.equalTo(self).offset(-8);
            make.size.equal.sizeOffset(CGSizeMake(controlButtonH, controlButtonH));
        }];
        currentLeftButton = self.nextButton;
    }
    else {
        [self.playButton bjl_remakeConstraints:^(BJLConstraintMaker *make) {
            make.left.equalTo(self).offset(margin);
            make.bottom.equalTo(self).offset(-8);
            make.size.equal.sizeOffset(CGSizeMake(controlButtonH, controlButtonH));
        }];
    }
    [self.currentTimeLabel bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        make.left.equalTo(currentLeftButton.bjl_right).offset(5.0);
        make.centerY.equalTo(currentLeftButton);
        make.horizontal.compressionResistance.hugging.required();
    }];
    [self updateAlbumButtonHidden:!(enable && self.horizen) && !self.externalAlnumShow];
}

- (void)updateAlbumButtonHidden:(BOOL)hidden {
    self.albumButton.hidden = hidden;
}

- (void)updateWithRate:(NSString *)rateString {
    CGFloat rate = rateString.floatValue;
    if (rate == 1) {
        rateString = BJLLocalizedString(@"倍速");
    }
    [self.rateButton setTitle:rateString ?: @"1.0X" forState:UIControlStateNormal];
}

- (void)updateWithDefinition:(NSString *)definitionString {
    [self.definitionButton setTitle:definitionString ?: BJLLocalizedString(@"高清") forState:UIControlStateNormal];
}

- (void)updateSubtitleExist:(BOOL)exist {
    self.existSubtitle = exist;
    [self updateConstraintsWithLayoutType:self.horizen];
}

- (void)setVideoRateEnable:(BOOL)enable {
    self.rateButtonEnable = enable;
    self.rateButton.hidden = !enable;
}

- (void)setScaleEnable:(BOOL)enable {
    self.scaleButtonEnable = enable;
    self.scaleButton.hidden = !enable;
}

// 外部控制展示时，优先级最高，一定展示
- (void)setExternalAlnumShow:(BOOL)show {
    _externalAlnumShow = show;
    [self updateAlbumButtonHidden:(self.preButton.hidden || !self.horizen) && !self.externalAlnumShow];
}

#pragma mark - actions

- (void)playAction:(UIButton *)button {
    if (self.mediaPlayCallback) {
        self.mediaPlayCallback();
    }

    [self disablePlayControlsAndThenRecover];
}

- (void)nextButtonOnClick:(UIButton *)button {
    if (self.nextMediaPlayCallback) {
        self.nextMediaPlayCallback();
    }
    [self disablePlayControlsAndThenRecover];

}

- (void)preButtonOnClick:(UIButton *)button {
    if (self.preMediaPlayCallback) {
        self.preMediaPlayCallback();
    }
    [self disablePlayControlsAndThenRecover];
}

- (void)pauseAction:(UIButton *)button {
    if (self.mediaPauseCallback) {
        self.mediaPauseCallback();
    }

    [self disablePlayControlsAndThenRecover];
}

- (void)scaleAction:(UIButton *)button {
    if (self.scaleCallback) {
        self.scaleCallback(!self.horizen);
    }
}

- (void)showSubtitleList {
    if (self.showSubtitleListCallback) {
        self.showSubtitleListCallback();
    }
}

- (void)showRateList {
    if (self.showRateListCallback) {
        self.showRateListCallback();
    }
}

- (void)showDefinitionList {
    if (self.showDefinitionListCallback) {
        self.showDefinitionListCallback();
    }
}

- (void)showAlbumList {
    self.albumButton.selected = !self.albumButton.selected;
    if (self.showAlbumListCallback) {
        self.showAlbumListCallback(self.albumButton.selected);
    }
}

- (void)sliderChanged:(BJPUVideoSlider *)slider {
    self.stopUpdateProgress = YES;
    self.slideCanceled = NO;
    if (slider.maximumValue > 0.0) {
        self.currentTimeLabel.text = [NSString stringFromTimeInterval:slider.value totalTimeInterval:slider.totalDuration];
    }
}

- (void)touchSlider:(BJPUVideoSlider *)slider {
    self.stopUpdateProgress = YES;
    self.slideCanceled = NO;
    if (slider.maximumValue > 0.0) {
        self.currentTimeLabel.text = [NSString stringFromTimeInterval:slider.value totalTimeInterval:slider.totalDuration];
    }
}

- (void)dragSlider:(BJPUVideoSlider *)slider {
    self.stopUpdateProgress = NO;
    self.slideCanceled = YES;
    if (self.mediaSeekCallback) {
        self.mediaSeekCallback(slider.value);
    }
}

- (void)disablePlayControlsAndThenRecover {
    [self setPlayControlButtonsEnabled:NO];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self setPlayControlButtonsEnabled:YES];
    });
}

- (void)setPlayControlButtonsEnabled:(BOOL)enabled {
    self.playButton.enabled = enabled;
    self.pauseButton.enabled = enabled;
    self.preButton.enabled = enabled;
    self.nextButton.enabled = enabled;
}

#pragma mark - setters & getters
- (UIButton *)preButton {
    if (!_preButton) {
        _preButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button bjl_setImage:[UIImage bjpu_imageNamed:@"album_pre"] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(preButtonOnClick:) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _preButton;
}

- (UIButton *)nextButton {
    if (!_nextButton) {
        _nextButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button bjl_setImage:[UIImage bjpu_imageNamed:@"album_next"] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(nextButtonOnClick:) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _nextButton;
}

- (UIButton *)playButton {
    if (!_playButton) {
        _playButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button setImage:[BJPUTheme playButtonImage] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(playAction:) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _playButton;
}

- (UIButton *)pauseButton {
    if (!_pauseButton) {
        _pauseButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button setImage:[BJPUTheme pauseButtonImage] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(pauseAction:) forControlEvents:UIControlEventTouchUpInside];
            button.hidden = YES;
            button;
        });
    }
    return _pauseButton;
}

- (UIButton *)scaleButton {
    if (!_scaleButton) {
        _scaleButton = ({
            UIButton *button = [[UIButton alloc] init];
            [button setImage:[BJPUTheme scaleButtonImage] forState:UIControlStateNormal];
            [button addTarget:self action:@selector(scaleAction:) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _scaleButton;
}

- (UILabel *)currentTimeLabel {
    if (!_currentTimeLabel) {
        _currentTimeLabel = ({
            UILabel *label = [[UILabel alloc] init];
            label.textColor = [BJPUTheme defaultTextColor];
            label.font = [UIFont systemFontOfSize:timeFontSize];
            label.adjustsFontSizeToFitWidth = YES;
            label;
        });
    }
    return _currentTimeLabel;
}

- (BJPUProgressView *)progressView {
    if (!_progressView) {
        _progressView = ({
            BJPUProgressView *view = [[BJPUProgressView alloc] init];
            [view.slider addTarget:self action:@selector(sliderChanged:) forControlEvents:UIControlEventValueChanged];
            [view.slider addTarget:self action:@selector(touchSlider:) forControlEvents:UIControlEventTouchDragInside];
            [view.slider addTarget:self action:@selector(dragSlider:) forControlEvents:UIControlEventTouchUpInside];
            [view.slider addTarget:self action:@selector(dragSlider:) forControlEvents:UIControlEventTouchUpOutside];
            [view.slider addTarget:self action:@selector(dragSlider:) forControlEvents:UIControlEventTouchCancel];
            view;
        });
    }
    return _progressView;
}

- (UIStackView *)buttonStack {
    if (!_buttonStack) {
        _buttonStack = [[UIStackView alloc] init];
        _buttonStack.axis = UILayoutConstraintAxisHorizontal;
        _buttonStack.distribution = UIStackViewDistributionFill;
        _buttonStack.alignment = UIStackViewAlignmentCenter;
        [_buttonStack setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
        _buttonStack.spacing = 5;
    }
    return _buttonStack;
}

- (UIButton *)subtitleButton {
    if (!_subtitleButton) {
        _subtitleButton = ({
            UIButton *button = [[UIButton alloc] init];
            button.clipsToBounds = YES;
            [button setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateNormal];
            [button.titleLabel setFont:[UIFont systemFontOfSize:14]];
            [button setTitle:BJLLocalizedString(@"字幕") forState:UIControlStateNormal];
            [button addTarget:self action:@selector(showSubtitleList) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _subtitleButton;
}

- (UIButton *)definitionButton {
    if (!_definitionButton) {
        _definitionButton = ({
            UIButton *button = [[UIButton alloc] init];
            button.clipsToBounds = YES;
            [button setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateNormal];
            [button.titleLabel setFont:[UIFont systemFontOfSize:14]];
            [button setTitle:BJLLocalizedString(@"清晰度") forState:UIControlStateNormal];
            [button addTarget:self action:@selector(showDefinitionList) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _definitionButton;
}

- (UIButton *)albumButton {
    if (!_albumButton) {
        _albumButton = ({
            UIButton *button = [[UIButton alloc] init];
            button.clipsToBounds = YES;
            [button bjl_setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateNormal possibleStates:UIControlStateHighlighted];
            [button bjl_setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateSelected possibleStates:UIControlStateHighlighted];
            [button.titleLabel setFont:[UIFont systemFontOfSize:14]];
            [button bjl_setTitle:BJLLocalizedString(@"选集") forState:UIControlStateNormal possibleStates:UIControlStateHighlighted];
            [button bjl_setTitle:BJLLocalizedString(@"选集") forState:UIControlStateSelected possibleStates:UIControlStateHighlighted];
            [button addTarget:self action:@selector(showAlbumList) forControlEvents:UIControlEventTouchUpInside];
            button.hidden = YES;
            button;
        });
    }
    return _albumButton;
}

- (UIButton *)rateButton {
    if (!_rateButton) {
        _rateButton = ({
            UIButton *button = [[UIButton alloc] init];
            button.clipsToBounds = YES;
            [button setTitleColor:[BJPUTheme defaultTextColor] forState:UIControlStateNormal];
            [button.titleLabel setFont:[UIFont systemFontOfSize:14]];
            [button setTitle:BJLLocalizedString(@"倍速") forState:UIControlStateNormal];
            [button addTarget:self action:@selector(showRateList) forControlEvents:UIControlEventTouchUpInside];
            button;
        });
    }
    return _rateButton;
}

@end
