//
//  BJPUSignInAlertViewController.m
//  BJVideoPlayerUI
//
//  Created by ney on 2023/10/18.
//  Copyright © 2023 BaijiaYun. All rights reserved.
//

#import "BJPUSignInAlertViewController.h"
#import "BJPUAppearance.h"

@interface BJPUSignInAlertView: UIView
@property (nonatomic, strong) BJPUSignInModel *signData;

@property (nonatomic, strong) UIImageView *titleIconView;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *messageLabel;
@property (nonatomic, strong) UIButton *signButton;

@property (nonatomic, copy) void (^_Nullable userDidSignInCallback)(BJPUSignInModel *signInData);

- (instancetype)initWithSignInData:(BJPUSignInModel *)signData;
@end


@interface BJPUSignInAlertViewController ()
@property (nonatomic, weak) BJVPlayerManager *player;
@property (nonatomic, copy) NSArray <BJPUSignInModel *> *signInList;
@property (nonatomic, copy) NSArray <BJPUSignInModel *> *sortedSignInList;
@property (nonatomic, strong) BJPUSignInAlertView *alertView;

@property (nonatomic, strong) NSMutableSet *didShownSignData;
@property (nonatomic, assign) BOOL shouldResumePlay;
@end


@implementation BJPUSignInAlertViewController
- (instancetype)initWithPlayer:(BJVPlayerManager *)player signInList:(NSArray <BJPUSignInModel *> *)signInList {
    if (self) {
        self.player = player;
        self.signInList = signInList;
        [self addObserver];
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.view.backgroundColor = [UIColor.blackColor colorWithAlphaComponent:0.2];
    UIGestureRecognizer *tap = [UITapGestureRecognizer bjl_gestureWithHandler:^(__kindof UIGestureRecognizer * _Nullable gesture) {
        // touch barrier
    }];
    [self.view addGestureRecognizer:tap];
}

- (void)addObserver {
    bjl_weakify(self);
    [self bjl_kvo:BJLMakeProperty(self.player, currentTime)
         observer:^BOOL(id now, id old, BJLPropertyChange * _Nullable change) {
        bjl_strongify(self);
        [self showAlertIfNeeded];
        return YES;
    }];
    
    [self bjl_kvo:BJLMakeProperty(self.player, duration)
         observer:^BOOL(id now, id old, BJLPropertyChange * _Nullable change) {
        bjl_strongify(self);
        
        if (self.player.duration > 0
            && self.sortedSignInList.count == 0
            && self.signInList.count > 0) {
            NSMutableArray *validData = [NSMutableArray array];
            for (BJPUSignInModel *i in self.signInList) {
                if (i.timeOffset >= 0 && i.timeOffset < self.player.duration) {
                    [validData addObject:i];
                }
            }
            if (validData.count > 0) {
                [validData sortUsingComparator:^NSComparisonResult(BJPUSignInModel   * _Nonnull obj1, BJPUSignInModel   * _Nonnull obj2) {
                    NSNumber *obj1Val = [NSNumber numberWithDouble:obj1.timeOffset];
                    NSNumber *obj2Val = [NSNumber numberWithDouble:obj2.timeOffset];
                    return [obj1Val compare:obj2Val];
                }];
                self.sortedSignInList = validData;
            }
        }
        return YES;
    }];
}

- (void)showAlertIfNeeded {
    // 按照 timeoffset 从小到大遍历：
    // 如果存在时间小于当前时间的签到，并且这个签到用户还没点击过，就弹出 alert， seek 到对应的时间点
    BJPUSignInModel *data = [self findDataByTimeOffset:self.player.currentTime];
    if (data == nil || self.alertView != nil) { return; }
    
    [self showAlertWithSignData:data];
    if (self.player.playStatus == BJVPlayerStatus_playing) {
        [self.player pause];
        self.shouldResumePlay = YES;
    }
    else {
        self.shouldResumePlay = NO;
    }
    [self.player seek:data.timeOffset];
}

/// 返回小于当前时间的第一个没有签到过的数据
/// - Parameter timeOffset: timeOffset
- (BJPUSignInModel *)findDataByTimeOffset:(NSTimeInterval)timeOffset {
    for (BJPUSignInModel *i in self.sortedSignInList) {
        if (i.timeOffset <= timeOffset && ![self signDataDidShown:i]) {
            return i;
        }
    }
    return nil;
}

- (BOOL)signDataDidShown:(BJPUSignInModel *)data {
    return [self.didShownSignData containsObject:data];
}

- (void)markSignDataAsShown:(BJPUSignInModel *)data {
    [self.didShownSignData addObject:data];
}

- (void)showAlertWithSignData:(BJPUSignInModel *)signData {
    if (self.parentView == nil) return;
    
    self.alertView = [[BJPUSignInAlertView alloc] initWithSignInData:signData];
    bjl_weakify(self);
    self.alertView.userDidSignInCallback = ^(BJPUSignInModel *signInData) {
        bjl_strongify(self);
        
        [self markSignDataAsShown:signData];
        [self hideAlert];
        if (self.shouldResumePlay) { [self.player play]; }
        
        if (self.userDidSignInCallback) {
            self.userDidSignInCallback(self, signData);
        }
    };
    [self.view addSubview:self.alertView];
    [self.alertView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.center.equalTo(self.view);
        make.width.lessThanOrEqualTo(self.view).multipliedBy(0.7);
    }];
    
    [self.view removeFromSuperview];
    [self.parentView addSubview:self.view];
    [self.view bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.edges.equalTo(self.parentView);
    }];
}

- (void)hideAlert {
    [self bjl_removeFromParentViewControllerAndSuperiew];
    [self.alertView removeFromSuperview];
    self.alertView = nil;
}

- (NSMutableSet *)didShownSignData {
    if(!_didShownSignData) {
        _didShownSignData = [[NSMutableSet alloc] init];
    }
    return _didShownSignData;
}
@end


@implementation BJPUSignInAlertView
- (instancetype)initWithSignInData:(BJPUSignInModel *)signData {
    self = [super init];
    if (self) {
        self.signData = signData;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.layer.masksToBounds = YES;
    self.layer.cornerRadius = 4;
    self.backgroundColor = UIColor.whiteColor;
    
    [self addSubview:self.titleIconView];
    [self addSubview:self.titleLabel];
    [self addSubview:self.messageLabel];
    [self addSubview:self.signButton];
    
    [self.titleIconView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.equalTo(@12);
        make.centerX.equalTo(self);
    }];
    [self.titleLabel bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.top.equalTo(self.titleIconView.bjl_bottom).offset(8);
        make.centerX.equalTo(self);
    }];
    [self.messageLabel bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.left.greaterThanOrEqualTo(self).offset(16);
        make.right.lessThanOrEqualTo(self).offset(-16);
        make.top.equalTo(self.titleLabel.bjl_bottom).offset(4);
        make.centerX.equalTo(self);
    }];
    [self.signButton bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        make.centerX.equalTo(self);
        make.left.greaterThanOrEqualTo(self).offset(16);
        make.right.lessThanOrEqualTo(self).offset(-16);
        make.top.equalTo(self.messageLabel.bjl_bottom).offset(20);
        make.bottom.equalTo(self).offset(-16);
        make.width.greaterThanOrEqualTo(@120);
        make.height.greaterThanOrEqualTo(@32);
    }];
    
    [self bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
        BOOL iPhone = (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone);
        make.width.greaterThanOrEqualTo(iPhone ? @320 : @320).priorityHigh();
        make.height.greaterThanOrEqualTo(iPhone ? @200 : @200).priorityHigh();
        
        make.width.equalTo(iPhone ? @320 : @320).priority(UILayoutPriorityDefaultLow);
        make.height.equalTo(iPhone ? @200 : @200).priority(UILayoutPriorityDefaultLow);
    }];
    
    if (self.signData.titleIconURLString.length > 0) {
        [self.titleIconView bjl_setImageWithURL:[NSURL URLWithString:self.signData.titleIconURLString] placeholder:nil completion:^(UIImage * _Nullable image, NSError * _Nullable error, NSURL * _Nullable imageURL) {
        }];
    }
        
    // * 标题：最大 16 字符
    // * 副标题：最大 48 字符
    // * 按钮：最大 20 字符
    if (self.signData.title.length > 0) {
        self.titleLabel.text = self.signData.title.length <= 16 ? self.signData.title : [self.signData.title substringWithRange:NSMakeRange(0, 16)];
    }
    if (self.signData.message.length > 0) {
        self.messageLabel.text = self.signData.message.length <= 48 ? self.signData.message : [self.signData.message substringWithRange:NSMakeRange(0, 48)];
    }
    if (self.signData.buttonText.length > 0) {
        NSString *buttonText = self.signData.buttonText.length <= 20 ? self.signData.buttonText : [self.signData.buttonText substringWithRange:NSMakeRange(0, 20)];
        [self.signButton setTitle:buttonText forState:UIControlStateNormal];
    }
}

- (UIImageView *)titleIconView {
    if(!_titleIconView) {
        _titleIconView = [[UIImageView alloc] init];
        _titleIconView.image = [UIImage bjpu_imageNamed:@"sign_in_alert"];
        _titleIconView.accessibilityIdentifier = @"_titleIconView";
        [_titleIconView bjl_makeConstraints:^(BJLConstraintMaker * _Nonnull make) {
            make.width.equalTo(@72);
            make.height.equalTo(@72);
        }];
    }
    return _titleIconView;
}

- (UILabel *)titleLabel {
    if(!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.font = [UIFont systemFontOfSize:14];
        _titleLabel.textColor = [UIColor bjl_colorWithHexString:@"#4A4A4A"];
        _titleLabel.text = BJLLocalizedString(@"学习签到");
        _titleLabel.accessibilityIdentifier = @"_titleLabel";
    }
    return _titleLabel;
}

- (UILabel *)messageLabel {
    if(!_messageLabel) {
        _messageLabel = [[UILabel alloc] init];
        _messageLabel.font = [UIFont systemFontOfSize:12];
        _messageLabel.numberOfLines = 0;
        _messageLabel.textColor = [UIColor bjl_colorWithHexString:@"#666666"];;
        _messageLabel.text = BJLLocalizedString(@"请点击签到，签到后可继续学习");
        _messageLabel.accessibilityIdentifier = @"_messageLabel";
    }
    return _messageLabel;
}

- (UIButton *)signButton {
    if(!_signButton) {
        _signButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [_signButton setTitle:BJLLocalizedString(@"签到") forState:UIControlStateNormal];
        _signButton.titleLabel.font = [UIFont systemFontOfSize:14];
        _signButton.titleLabel.textColor = UIColor.whiteColor;
        _signButton.accessibilityIdentifier = @"_signButton";
        _signButton.layer.masksToBounds = YES;
        _signButton.layer.cornerRadius = 3;
        [_signButton setBackgroundColor:[UIColor bjl_colorWithHexString:@"#315cf6"]];
        [_signButton setContentEdgeInsets:UIEdgeInsetsMake(0, 10, 0, 10)];
        
        bjl_weakify(self);
        [_signButton bjl_addHandler:^(UIButton * _Nonnull button) {
            bjl_strongify(self);
            if (self.userDidSignInCallback) {
                self.userDidSignInCallback(self.signData);
            }
        }];
    }
    return _signButton;
}
@end
